@@ -995,9 +995,12 @@ parse_resourcetype (xmlNodePtr rt)
 }
 
 static inline void
-file_info_set_content_type (GFileInfo *info, const char *type)
+file_info_set_content_type (GFileInfo *info,
+			    const char *type,
+			    gboolean uncertain_content_type)
 {
-  g_file_info_set_content_type (info, type);
+  if (!uncertain_content_type)
+    g_file_info_set_content_type (info, type);
   g_file_info_set_attribute_string (info,
                                     G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
                                     type);
@@ -1014,9 +1017,9 @@ ms_response_to_file_info (MsResponse *response,
   guint       status;
   char       *basename;
   const char *text;
-  GTimeVal    tv;
   GFileType   file_type;
   char       *mime_type;
+  gboolean    uncertain_content_type;
   GIcon      *icon;
   GIcon      *symbolic_icon;
   gboolean    have_display_name;
@@ -1032,6 +1035,7 @@ ms_response_to_file_info (MsResponse *response,
 
   file_type = G_FILE_TYPE_REGULAR;
   mime_type = NULL;
+  uncertain_content_type = FALSE;
 
   have_display_name = FALSE;
   ms_response_get_propstat_iter (response, &iter);
@@ -1065,12 +1069,15 @@ ms_response_to_file_info (MsResponse *response,
             }
           else if (node_has_name (node, "creationdate"))
             {
-              if (! g_time_val_from_iso8601 (text, &tv))
+              GDateTime *dt;
+
+              if ((dt = g_date_time_new_from_iso8601 (text, NULL)) == NULL)
                 continue;
 
               g_file_info_set_attribute_uint64 (info,
                                                 G_FILE_ATTRIBUTE_TIME_CREATED,
-                                                tv.tv_sec);
+                                                g_date_time_to_unix (dt));
+              g_date_time_unref (dt);
             }
           else if (node_has_name (node, "getcontenttype"))
             {
@@ -1096,12 +1103,12 @@ ms_response_to_file_info (MsResponse *response,
           else if (node_has_name (node, "getlastmodified"))
             {
               SoupDate *sd;
-              GTimeVal tv;
+
 	      sd = soup_date_new_from_string(text);
 	      if (sd)
 	        {
-		  soup_date_to_timeval (sd, &tv);
-		  g_file_info_set_modification_time (info, &tv);
+                  g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, soup_date_to_time_t (sd));
+                  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, 0);
 		  soup_date_free (sd);
 		}
 	    }
@@ -1116,7 +1123,7 @@ ms_response_to_file_info (MsResponse *response,
 
       icon = g_content_type_get_icon (mime_type);
       symbolic_icon = g_content_type_get_symbolic_icon (mime_type);
-      file_info_set_content_type (info, mime_type);
+      file_info_set_content_type (info, mime_type, FALSE);
 
       /* Ignore file size for directories. Most of the servers don't report it
        * for directories anyway. However, some servers report total size of
@@ -1128,7 +1135,7 @@ ms_response_to_file_info (MsResponse *response,
   else
     {
       if (mime_type == NULL)
-        mime_type = g_content_type_guess (basename, NULL, 0, NULL);
+        mime_type = g_content_type_guess (basename, NULL, 0, &uncertain_content_type);
 
       icon = g_content_type_get_icon (mime_type);
       if (G_IS_THEMED_ICON (icon))
@@ -1142,7 +1149,7 @@ ms_response_to_file_info (MsResponse *response,
           g_themed_icon_append_name (G_THEMED_ICON (symbolic_icon), "text-x-generic-symbolic");
         }
 
-      file_info_set_content_type (info, mime_type);
+      file_info_set_content_type (info, mime_type, uncertain_content_type);
     }
 
   if (have_display_name == FALSE)
@@ -1761,11 +1768,13 @@ dav_uri_from_dns_sd_resolver (GVfsBackendDav *dav_backend)
   char       *user;
   char       *path;
   char       *address;
+  gchar      *interface;
   const char *service_type;
   guint       port;
 
   service_type = g_vfs_dns_sd_resolver_get_service_type (dav_backend->resolver);
   address = g_vfs_dns_sd_resolver_get_address (dav_backend->resolver);
+  interface = g_vfs_dns_sd_resolver_get_interface (dav_backend->resolver);
   port = g_vfs_dns_sd_resolver_get_port (dav_backend->resolver);
   user = g_vfs_dns_sd_resolver_lookup_txt_record (dav_backend->resolver, "u"); /* mandatory */
   path = g_vfs_dns_sd_resolver_lookup_txt_record (dav_backend->resolver, "path"); /* optional */
@@ -1785,7 +1794,20 @@ dav_uri_from_dns_sd_resolver (GVfsBackendDav *dav_backend)
 
   soup_uri_set_port (uri, port);
 
-  soup_uri_set_host (uri, address);
+  /* IPv6 host does not include brackets in SoupURI, but GVfsDnsSdResolver host does */
+  if (gvfs_is_ipv6 (address))
+    {
+      /* Link-local addresses require interface to be specified. */
+      if (g_str_has_prefix (address, "[fe80:") && interface != NULL)
+        {
+          uri->host = g_strconcat (address + 1, interface, NULL);
+          uri->host[strlen (address) - 2] = '%';
+        }
+      else
+        uri->host = g_strndup (address + 1, strlen (address) - 2);
+    }
+  else
+    soup_uri_set_host (uri, address);
 
   if (path != NULL)
     soup_uri_set_path (uri, path);
@@ -1794,6 +1816,7 @@ dav_uri_from_dns_sd_resolver (GVfsBackendDav *dav_backend)
 
 
   g_free (address);
+  g_free (interface);
   g_free (user);
   g_free (path);
 
